#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import random
import os
import sys


colors = {
    "black": "\u001b[30m",
    "red": "\u001b[31m",
    "green": "\u001b[32m",
    "yellow": "\u001b[33m",
    "blue": "\u001b[34m",
    "magenta": "\u001b[35m",
    "cyan": "\u001b[36m",
    "white": "\u001b[37m",
    "reset": "\u001b[0m",

    "b_black": "\u001b[30;1m",
    "b_red": "\u001b[31;1m",
    "b_green": "\u001b[32;1m",
    "b_yellow": "\u001b[33;1m",
    "b_blue": "\u001b[34;1m",
    "b_magenta": "\u001b[35;1m",
    "b_cyan": "\u001b[36;1m",
    "b_white": "\u001b[37;1m",

    "Reset": "\u001b[0m",
}

def Colored(string, color):
    return string if "-m" in sys.argv else ( colors[color] + string + colors["Reset"])


def Clear_Screen():
    print("\033[2J") # erase saved lines
    print("\033[3J") # erase entire screen
    print("\033[H") # moves cursor to home position


def Get_Arguments():
    args_dict = {"colors": []}
    if sys.argv:
        for arg in sys.argv:
            if "-i=" in arg:
                args_dict["intensity"] = int(arg.split("-i=")[1])
            if "-t=" in arg:
                args_dict["timing"] = float(arg.split("-t=")[1])
            if arg in colors:
                args_dict["colors"].append(arg)
    if not args_dict["colors"]:
        del args_dict["colors"]
    return args_dict


def New_Drop():
    for i in range(intensity):
        shape = random.choice(DROPSHAPES)
        color = random.choice(drop_colors)

        raindrop = {
            "shape": Colored(shape, color),
            "x": random.randint(0, xmax),
            "y": 0,
        }
        rainfall.append(raindrop)

def Rain():
    ## iterate over every line
    for i in range(ymax):
        line = " "*xmax

        ### to avoid splicing of ansi codes, splice in the drops from the end of the line
        this_line_raindrops = [raindrop for raindrop in rainfall if raindrop["y"] == i]
        this_line_raindrops.sort(key=lambda y: y["x"])
        this_line_raindrops.reverse()

        ## insert new drops and shift existing drops
        for raindrop in this_line_raindrops:
            x = raindrop["x"]
            line = line[:x] + raindrop["shape"] +line[x:]

        print(line)

    ### update raindrop positions
    for raindrop in rainfall:
        raindrop["y"] += 1

        ## once a raindrop reaches the ground, they splash
        if raindrop["y"] > ymax-2:
            raindrop["shape"] = Colored("o", random.choice(drop_colors))

        # raindrops outside the window evaporate
        if raindrop["y"] > ymax:
            rainfall.remove(raindrop)

    New_Drop()



def Weather_Forecast():
    global weather
    global intensity

    weather += 1
    if weather == 100:
        weather = 0
        intensity += random.choice([-1,1])
        if intensity < 1:
            intensity = 1
        if intensity > 10:
            intensity = 10


size = os.get_terminal_size()
xmax = size.columns
ymax = int(size.lines)

weather = 0
rainfall = []
DROPSHAPES =["|", "│", "┃", "╽", "╿", "║", "┆", "┇", "┊", "┋", "╵", "╹", "╻"]

args = Get_Arguments()
intensity = args.get("intensity", 1)
timing = args.get("timing", 0.08)
drop_colors = args.get("colors", ["blue", "b_blue"])



print('\033[?25l', end="") ## hides the cursor
New_Drop()

try:
    while True:
        Rain()
        time.sleep(timing)
        Clear_Screen()
        Weather_Forecast()

except KeyboardInterrupt:
    Clear_Screen()
    print('\033[?25h', end="") # makes cursor visible again




















import gi
import subprocess

gi.require_version('Gtk', '3.0')
gi.require_version('AppIndicator3', '0.1')
gi.require_version('GLib', '2.0')
from gi.repository import Gtk, AppIndicator3, GLib

class playerctl_systray:
    def __init__(self):
        self.last_instances = self.get_player_instances()
        self.indicator = AppIndicator3.Indicator.new(
            "media-control-app",
            "audio-x-generic",
            AppIndicator3.IndicatorCategory.APPLICATION_STATUS
        )
        self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
        self.indicator.set_menu(self.create_menu())

    def create_menu(self):
        menu = Gtk.Menu()

        instances = self.get_player_instances()
        if instances:
            for instance in instances:
                instance_submenu = Gtk.Menu()

                play_item = Gtk.MenuItem(label="play-pause")
                play_item.connect("activate", self.play_pause, instance)
                instance_submenu.append(play_item)

                next_item = Gtk.MenuItem(label="next")
                next_item.connect("activate", self.next_track, instance)
                instance_submenu.append(next_item)

                prev_item = Gtk.MenuItem(label="previous")
                prev_item.connect("activate", self.prev_track, instance)
                instance_submenu.append(prev_item)

                instance_menu_item = Gtk.MenuItem(label=instance)
                instance_menu_item.set_submenu(instance_submenu)
                menu.append(instance_menu_item)

            a_submenu = Gtk.Menu()

            a_pause_item = Gtk.MenuItem(label="pause")
            a_pause_item.connect("activate", self.a_pause)
            a_submenu.append(a_pause_item)

            a_play_item = Gtk.MenuItem(label="play")
            a_play_item.connect("activate", self.a_play)
            a_submenu.append(a_play_item)

            a_next_item = Gtk.MenuItem(label="next")
            a_next_item.connect("activate", self.a_next_track)
            a_submenu.append(a_next_item)

            a_prev_item = Gtk.MenuItem(label="previous")
            a_prev_item.connect("activate", self.a_prev_track)
            a_submenu.append(a_prev_item)

            a_menu_item = Gtk.MenuItem(label="-a")
            a_menu_item.set_submenu(a_submenu)
            menu.append(a_menu_item)
        else:
            no_players_item = Gtk.MenuItem(label="no players found")
            no_players_item.set_sensitive(False)  # greyed out
            menu.append(no_players_item)

        menu.append(Gtk.SeparatorMenuItem())  # separator

        quit_item = Gtk.MenuItem(label="quit")
        quit_item.connect("activate", self.quit)
        menu.append(quit_item)

        menu.show_all()
        return menu

    def get_player_instances(self):
        result = subprocess.run("playerctl -l", shell=True, capture_output=True, text=True)
        return result.stdout.strip().split('\n') if result.stdout.strip() else []

    def play_pause(self, _, instance):
        self.run_command(f"playerctl -p {instance} play-pause")

    def next_track(self, _, instance):
        self.run_command(f"playerctl -p {instance} next")

    def prev_track(self, _, instance):
        self.run_command(f"playerctl -p {instance} previous")

    def a_play(self, _):
        self.run_command("playerctl -a play")

    def a_pause(self, _):
        self.run_command("playerctl -a pause")

    def a_next_track(self, _):
        self.run_command("playerctl -a next")

    def a_prev_track(self, _):
        self.run_command("playerctl -a previous")

    def run_command(self, command):
        subprocess.run(command, shell=True)

    def quit(self, _):
        Gtk.main_quit()

    def refresh_menu(self):
        current_instances = self.get_player_instances()
        if current_instances != self.last_instances:
            self.indicator.set_menu(self.create_menu())
            self.last_instances = current_instances
        return True  # returns True so that timeout continues

def main():
    app = playerctl_systray()
    GLib.timeout_add(1024, app.refresh_menu)  # probably should be event-based in the future
    Gtk.main()

if __name__ == "__main__":
    main()
